package dm8xx;

import java.util.AbstractQueue;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;

public final class BPriorityQueue<E> extends AbstractQueue<E>
{
	private Object[] heap; // 1-indexed.
	private int size;


	public BPriorityQueue()
	{
		this(11);
	}
	
	
	public BPriorityQueue(int initialCapacity)
	{
		if(initialCapacity < 1) {
			throw new IllegalArgumentException("initialCapacity < 1");
		}
		
		heap = new Object[initialCapacity + 1];
		size = 0;
	}
	

	public BPriorityQueue(Collection<? extends E> c)
	{
		throw new UnsupportedOperationException();
	}
	
	
	public BPriorityQueue(int initialCapacity,
			Comparator<? super E> comparator)
	{
		throw new UnsupportedOperationException();
	}
	
	
	public Comparator<? super E> comparator()
	{
		throw new UnsupportedOperationException();
	}


	@Override
	public Iterator<E> iterator()
	{
		throw new UnsupportedOperationException();
	}


	@Override
	public int size()
	{
		return size;
	}


	@Override
	public boolean offer(E element)
	{
		if(element == null) {
			throw new NullPointerException();
		}
		
		size++;
		
		if(size == heap.length) {
			expand();
		}
		
		heap[size] = element;
		siftUp(size);

		return true;
	}


	@Override
	public E peek()
	{		
		return (E) heap[1];
	}


	@Override
	public E poll()
	{
		if(size == 0) {
			return null;
		}

		E min = (E) heap[1];
		heap[1] = heap[size];
		heap[size] = null;
		size--;

		siftDown(1);

		return min;
	}
	
	
	private void expand()
	{
		// TODO: Check for overflow.
		
		Object[] newHeap = new Object[2 * (heap.length - 1) + 1];
		System.arraycopy(heap, 0, newHeap, 0, heap.length);
		heap = newHeap;
	}


	private void siftUp(int index)
	{
		Comparable<? super E> key = (Comparable<? super E>) heap[index];
		int parent = (int) (index / 2.0f);

		while(index > 1 && key.compareTo((E) heap[parent]) < 0) {
			Object temp = heap[index];
			heap[index] = heap[parent];
			heap[parent] = temp;

			index = parent;
			parent = (int) (index / 2.0f);
		}
	}


	private void siftDown(int index)
	{
		do {
			int left = 2 * index;
			int right = left + 1;
			int min = index;
	
			if(left <= size) {
				Comparable<? super E> key = (Comparable<? super E>) heap[left];
	
				if(key.compareTo((E) heap[index]) < 0) {
					min = left;
				}
			}
	
			if(right <= size) {
				Comparable<? super E> key = (Comparable<? super E>) heap[right];
	
				if(key.compareTo((E) heap[min]) < 0) {
					min = right;
				}
			}
			
			if(min == index) {
				break;
			}
	
			Object temp = heap[index];
			heap[index] = heap[min];
			heap[min] = temp;
			
			index = min;
		} while(true);
	}
}
