package dm8xx;

import java.util.AbstractQueue;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;

public final class TPriorityQueue<E> extends AbstractQueue<E>
{
	private Object[] heap; // 0-indexed.
	private int size;


	public TPriorityQueue()
	{
		this(11);
	}
	
	
	public TPriorityQueue(int initialCapacity)
	{
		if(initialCapacity < 1) {
			throw new IllegalArgumentException("initialCapacity < 1");
		}
		
		heap = new Object[initialCapacity];
		size = 0;
	}
	

	public TPriorityQueue(Collection<? extends E> c)
	{
		throw new UnsupportedOperationException();
	}
	
	
	public TPriorityQueue(
			int initialCapacity,
			Comparator<? super E> comparator)
	{
		throw new UnsupportedOperationException();
	}
	
	
	public Comparator<? super E> comparator()
	{
		throw new UnsupportedOperationException();
	}


	@Override
	public Iterator<E> iterator()
	{
		throw new UnsupportedOperationException();
	}


	@Override
	public int size()
	{
		return size;
	}


	@Override
	public boolean offer(E element)
	{
		if(element == null) {
			throw new NullPointerException();
		}
		
		if(size + 1 == heap.length) {
			expand();
		}
		
		heap[size] = element;
		siftUp(size);
		
		size++;

		return true;
	}


	@Override
	public E peek()
	{		
		return (E) heap[0];
	}


	@Override
	public E poll()
	{
		if(size == 0) {
			return null;
		}

		E min = (E) heap[0];
		heap[0] = heap[size - 1];
		heap[size - 1] = null;
		size--;

		siftDown(0);

		return min;
	}
	
	
	private void expand()
	{
		// TODO: Check for overflow.
		
		Object[] newHeap = new Object[2 * heap.length];
		System.arraycopy(heap, 0, newHeap, 0, heap.length);
		heap = newHeap;
	}


	private void siftUp(int index)
	{
		Comparable<? super E> key = (Comparable<? super E>) heap[index];
		int parent = (index - 1) / 3;

		while(index > 0 && key.compareTo((E) heap[parent]) < 0) {
			Object temp = heap[index];
			heap[index] = heap[parent];
			heap[parent] = temp;

			index = parent;
			parent = (index - 1) / 3;
		}
	}


	private void siftDown(int index)
	{
		do {
			int left = 3 * index + 1;
			int middle = left + 1;
			int right = middle + 1;
			int min = index;
	
			if(left < size) {
				Comparable<? super E> key = (Comparable<? super E>) heap[left];
	
				if(key.compareTo((E) heap[index]) < 0) {
					min = left;
				}
			}
			
			if(middle < size) {
				Comparable<? super E> key =
					(Comparable<? super E>) heap[middle];
	
				if(key.compareTo((E) heap[min]) < 0) {
					min = middle;
				}
			}
	
			if(right < size) {
				Comparable<? super E> key = (Comparable<? super E>) heap[right];
	
				if(key.compareTo((E) heap[min]) < 0) {
					min = right;
				}
			}
			
			if(min == index) {
				break;
			}
	
			Object temp = heap[index];
			heap[index] = heap[min];
			heap[min] = temp;
			
			index = min;
		} while(true);
	}
}
