package dm8xx.benchmark;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Random;

import bb.util.Benchmark;
import dm8xx.IntArrayList;
import dm8xx.IntegerArrayList;

public class ArrayListBenchmark extends AbstractBenchmark
{
	// Specify for which N's a benchmark should be run.
	private static final int[] N = new int[] {
		75000,
		125000,
		250000,
		500000,
		1000000,
		2000000,
		4000000
	};

	// Names of task classes (instantiated by using reflection).
	private static final String[] tasks = new String[] {
		"ArrayListOfIntegerTask",
		"IntegerArrayListTask",
		"IntArrayListTask"
	};


	public static void main(String[] args)
	throws ClassNotFoundException, IllegalArgumentException, SecurityException,
	InstantiationException, IllegalAccessException, InvocationTargetException
	{
		String[][] results = new String[tasks.length][N.length];

		for(int i = 0; i < tasks.length; i++) {
			Class<?> taskClass = Class.forName(
					"dm8xx.benchmark.ArrayListBenchmark$" + tasks[i]);

			for(int j = 0; j < N.length; j++) {
				Runnable task =
					(Runnable) taskClass.getConstructors()[0].newInstance(N[j]);

				System.gc();

				results[i][j] = new Benchmark(task).toString();
				printResult(tasks[i], N[j], results[i][j]);
			}
		}

		printAllResults(tasks, N, results);
	}


	private static abstract class Task implements Runnable
	{
		protected final int limit;
		protected final int[] ints;
		protected int result;


		protected Task(int limit)
		{
			this.limit = limit;

			ints = new int[limit];
			Random random = new Random(42);

			for(int i = 0; i < limit; i++)
			{
				ints[i] = random.nextInt();
			}
		}


		@Override
		public String toString()
		{
			// Avoid dead-code elimination.
			return "" + result;
		}
	}


	private static class ArrayListOfIntegerTask extends Task
	{
		private final ArrayList<Integer> arrayListOfInteger;


		public ArrayListOfIntegerTask(int limit)
		{
			super(limit);
			arrayListOfInteger = new ArrayList<Integer>(limit);
		}


		@Override
		public void run()
		{
			arrayListOfInteger.clear();

			for(int i = 0; i < limit; i++) {
				arrayListOfInteger.add(ints[i]);
			}

			for(int i = 0; i < limit; i++) {
				result += arrayListOfInteger.get(i);
			}
		}
	}


	private static class IntegerArrayListTask extends Task
	{
		private final IntegerArrayList integerArrayList;


		public IntegerArrayListTask(int limit)
		{
			super(limit);
			this.integerArrayList = new IntegerArrayList(limit);
		}


		@Override
		public void run()
		{
			integerArrayList.clear();

			for(int i = 0; i < limit; i++) {
				integerArrayList.add(ints[i]);
			}

			for(int i = 0; i < limit; i++) {
				result += integerArrayList.get(i);
			}
		}
	}


	private static class IntArrayListTask extends Task
	{
		private final IntArrayList intArrayList;


		public IntArrayListTask(int limit)
		{
			super(limit);
			this.intArrayList = new IntArrayList(limit);
		}


		@Override
		public void run()
		{
			intArrayList.clear();

			for(int i = 0; i < limit; i++) {
				intArrayList.add(ints[i]);
			}

			for(int i = 0; i < limit; i++) {
				result += intArrayList.get(i);
			}
		}
	}
}
