package dm8xx.benchmark;

import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Queue;
import java.util.Random;

import bb.util.Benchmark;

public class PriorityQueueBenchmark extends AbstractBenchmark
{
	// Specify for which N's a benchmark should be run.
	private static final int[] N = new int[] {
		500000,
		1000000,
		2000000,
		4000000
	};

	// Names of priority queue classes (instantiated by using reflection).
	private static final String[] queues = new String[] {
		"java.util.PriorityQueue",
		"dm8xx.BPriorityQueue",
		"dm8xx.TPriorityQueue"
	};


	@SuppressWarnings("unchecked")
	public static void main(String[] args)
	throws ClassNotFoundException, IllegalArgumentException, SecurityException,
	InstantiationException, IllegalAccessException, InvocationTargetException,
	NoSuchMethodException
	{
		String[][] results = new String[queues.length][N.length];

		for(int i = 0; i < queues.length; i++) {
			Class<?> taskClass = Class.forName(queues[i]);

			for(int j = 0; j < N.length; j++) {
				Queue<Integer> queue =
					(Queue<Integer>) taskClass.getConstructor(
							new Class<?>[] { int.class }).newInstance(N[j]);
				Runnable task = new QueueOfIntegerTask(queue, N[j]);

				System.gc();

				results[i][j] = new Benchmark(task).toString();
				printResult(queues[i], N[j], results[i][j]);
			}
		}

		printAllResults(queues, N, results);
	}


	private static class QueueOfIntegerTask implements Runnable
	{
		private final Queue<Integer> queue;
		private final int limit;
		private final Integer[] integers;
		private int result;


		public QueueOfIntegerTask(Queue<Integer> queue, int limit)
		{
			this.queue = queue;
			this.limit = limit;

			integers = new Integer[limit];

			for(int i = 0; i < limit; i++)
			{
				integers[i] = Integer.valueOf(i);
			}

			Collections.shuffle(Arrays.asList(integers), new Random(42));
		}


		@Override
		public String toString()
		{
			// Avoid dead-code elimination.
			return "" + result;
		}


		@Override
		public void run()
		{
			for(int i = 0; i < limit; i++) {
				queue.offer(integers[i]);
			}

			for(int i = 0; i < limit; i++) {
				result += queue.poll();
			}
		}
	}
}
